import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { IdpFormModel } from 'src/app/shared/model/idp-form.model';
import { EmployeeService } from 'src/app/shared/services/employee.service';
import { EvaluationIdpService } from 'src/app/shared/services/evaluation-Idp.service';
import Swal from 'sweetalert2';
@Component({
  selector: 'app-idp-evalution',
  templateUrl: './idp-evalution.component.html',
  styleUrls: ['./idp-evalution.component.scss']
})
export class IdpEvalutionComponent {
  @Input() pathTitle: string[] = []
  @Input() evaluationRoundId = ""
  @Input() evaluateeId = ""
  @Input() evaluaterId = ""
  @Output() sendReturnPath: EventEmitter<any> = new EventEmitter<any>();
  currentDate = new Date()
  hoveredCode: string | null = null;
  formIdp: { loading: boolean, data?: IdpFormModel } = { loading: false, data: undefined }
  canEdit = false
  constructor(private evaluationIdpService: EvaluationIdpService,
    private employeeService: EmployeeService,
    private cdr: ChangeDetectorRef
  ) {
  }
  ngOnInit(): void {
    this.getFormIdp()
  }
  getFormIdp() {
    this.formIdp.loading = true
    this.evaluationIdpService.getFormIdp(this.evaluationRoundId, this.evaluateeId).subscribe({
      next: response => {
        this.formIdp.data = JSON.parse(JSON.stringify(response))
        console.log(" 🐒 this.formIdp.data:", this.formIdp.data?.competencyIndicatorsCourses1)
        switch (this.formIdp.data?.currentStep) {
          case ("0"): {
            this.formIdp.data.masfromEvaluationIdp.apsassessyDate = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsassessy.employeeId
            break
          }
          case ("1"): {
            this.formIdp.data.masfromEvaluationIdp.apsapprove1Date = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsapprove1.employeeId
            break
          }
          case ("2"): {
            this.formIdp.data.masfromEvaluationIdp.apsapprove2Date = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsapprove2.employeeId
            break
          }
          case ("3"): {
            this.formIdp.data.masfromEvaluationIdp.apsapprove3Date = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsapprove3.employeeId
            break
          }
          case ("4"): {
            this.formIdp.data.masfromEvaluationIdp.apsapprove4Date = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsapprove4.employeeId
            break
          }
          case ("5"): {
            this.formIdp.data.masfromEvaluationIdp.apsapprove5Date = this.currentDate.toISOString().split('T')[0];
            this.canEdit = this.evaluaterId == this.formIdp.data.apsapprove5.employeeId
            break
          }
          default: { return }
        }
        this.formIdp.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.formIdp.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  returnPath() {
    this.sendReturnPath.emit()
  }
  convertDate(dateInput?: string | Date): string {
    let date = new Date()
    if (dateInput) {
      if (typeof dateInput === 'string') {
        const [year, month, day] = dateInput.split('-').map(Number);
        date = new Date(year, month - 1, day);
      } else {
        date = dateInput
      }
    }
    return date?.toLocaleDateString('th-TH', { day: 'numeric', month: 'long', year: 'numeric' }) || ''
  }
  convertDateFormat(dateString: string): string {
    const [year, month, day] = dateString.split("-");
    return `${day}-${month}-${year}`;
  }

  save(status?: string) {
    Swal.fire({
      iconHtml: `
              <div class="flex items-center justify-center rounded-full !h-80px !w-80px" style="background-color: #E8F8EE;">
                  <svg width="39" height="39" viewBox="0 0 39 39" fill="none" xmlns="http://www.w3.org/2000/svg">
                      <rect width="39" height="39" fill="#D2D2D2"/>
                      <g id="Component">
                          <g id="alert cart">
                              <g id="mdi:file-export">
                                  <circle cx="22.5" cy="19.5" r="33.5" fill="#E8F8EE"/>
                                  <path d="M9.75 3.25C8.88805 3.25 8.0614 3.59241 7.4519 4.2019C6.84241 4.8114 6.5 5.63805 6.5 6.5V32.5C6.5 33.362 6.84241 34.1886 7.4519 34.7981C8.0614 35.4076 8.88805 35.75 9.75 35.75H29.25C30.112 35.75 30.9386 35.4076 31.5481 34.7981C32.1576 34.1886 32.5 33.362 32.5 32.5V13L22.75 3.25M21.125 5.6875L30.0625 14.625H21.125M14.5113 19.8575H26V31.3463L22.555 27.9013L17.9563 32.5L13.3575 27.9013L17.9563 23.3188" 
                                  fill="#1DBE5A"/>
                              </g>
                          </g>
                      </g>
                  </svg>
              </div>
              `,
      title: status == 'draft' ? 'บันทึกแบบร่าง' : 'บันทึกข้อมูล',
      text: status == 'draft' ? 'คุณต้องการบันทึกแบบร่างของการประเมินนี้ใช่หรือไม่' : 'คุณต้องการบันทึกข้อมูลการประเมินนี้ใช่หรือไม่',
      showCancelButton: true,
      confirmButtonText: 'ยืนยันการบันทึก',
      cancelButtonText: 'ย้อนกลับ',
      customClass: {
        title: '!swal2-title-mt-20px',
        actions: '!swal2-actions-mt-20px',
        icon: '!swal2-icon-no-border',
        confirmButton: '!swal2-button-bg-green',
        cancelButton: '!swal2-button-bg-gray',
      },
    }).then((result) => {
      if (result.isConfirmed && this.formIdp) {
        this.saveApi(status)
      } else if (result.dismiss === Swal.DismissReason.cancel) {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        })
      }
    })
  }
  saveApi(status?: string) {
    let body = this.formIdp.data
    switch (this.formIdp.data?.currentStep) {
      case ("0"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsassessyStatus: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      case ("1"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove1Status: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      case ("2"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove2Status: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      case ("3"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove3Status: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      case ("4"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove4Status: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      case ("5"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove5Status: status == 'draft' ? '1' : '2'
          }
        }
        break
      }
      default: { return }
    }
    this.evaluationIdpService.postIdp(body).subscribe({
      next: response => {
        if (response.success) {
          Swal.fire({
            title: 'บันทึกสำเร็จ!',
            text: 'การประเมินของคุณถูกบันทึกแล้ว',
            icon: 'success',
            customClass: {
              confirmButton: '!swal2-button-bg-green',
            }
          });
          this.getFormIdp()
        } else {
          Swal.fire({
            title: 'ยกเลิก!',
            text: 'การบันทึกถูกยกเลิก',
            icon: 'error',
            customClass: {
              confirmButton: '!swal2-button-bg-danger',
            }
          });
        }
      }, error: error => {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        });
      }
    })
  }
}
